#! /usr/bin/env python
import click

import glob

from simtk import unit as u
from simtk.openmm import LangevinIntegrator, Platform, XmlSerializer
from simtk.openmm.app import Simulation, StateDataReporter

from openmmsystems.reporters import HDF5Reporter
from openmmsystems import OpenMMToolsTestSystem


def next_sequence():
    state_files = glob.glob("state*.xml")
    if len(state_files) == 0:
        return 0
    state_ids = [int(f.replace("state","").replace(".xml", "")) for f in state_files]
    return max(state_ids) + 1

@click.command()
@click.option("--platform", "-p", default="CUDA")
#@click.option("--state", "-s", type=click.Path(exists=True), default=None)
@click.option("--temperature", "-t", type=float, default=300.0)
@click.option("--ns", "-n", type=float, default=100)
def main(platform, temperature, ns):

    sequence = next_sequence()

    # settings
    timestep = 2*u.femtosecond
    length = ns*u.nanosecond
    n_steps = length / timestep

    # system and simulation
    bpti = OpenMMToolsTestSystem("AlanineDipeptideImplicit")
    platform = Platform.getPlatformByName(platform)
    platform_properties = {"DeviceIndex":'0', 'Precision': 'mixed'} if platform == "CUDA" else {}
    simulation = Simulation(
        bpti.topology,
        bpti.system,
        LangevinIntegrator(temperature*u.kelvin, 1./u.picosecond, timestep),
        platform,
        platform_properties,
    )
    simulation.reporters.append(StateDataReporter(
        f"data{sequence}.txt", 5000,
        step=True, time=True, potentialEnergy=True,
        kineticEnergy=True, temperature=True,
        volume=True, density=True)
    )
    simulation.reporters.append(HDF5Reporter(f"traj{sequence}.h5", 5000, forces=True))

    # restart
    print(f"Starting from Sequence {sequence}.")
    ctx = simulation.context
    if sequence > 0:
        simulation.loadState(f"state{sequence-1}.xml")
    else:
        ctx = simulation.context
        ctx.setPositions(bpti.positions)
        ctx.setVelocitiesToTemperature(temperature*u.kelvin)

    # run
    print(f"Running {n_steps} steps")
    simulation.step(n_steps+5000)

    # checkpoint
    ctx.getState(getPositions=True, getVelocities=True)
    simulation.saveState(f"state{sequence}.xml")


if __name__ == "__main__":
    main()

